@@ -2,7 +2,7 @@
  * DMI Decode
  *
  *   (C) 2000-2002 Alan Cox <alan@redhat.com>
- *   (C) 2002-2003 Jean Delvare <khali@linux-fr>
+ *   (C) 2002-2004 Jean Delvare <khali@linux-fr.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -47,14 +47,8 @@
  *    http://www.amd.com/us-en/assets/content_type/white_papers_and_tech_docs/20734.pdf
  */
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#ifdef USE_MMAP
-#include <sys/mman.h>
-#endif /* USE MMAP */
 #include <stdio.h>
 #include <unistd.h>
-#include <fcntl.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -69,71 +63,6 @@
 static const char *out_of_spec = "<OUT OF SPEC>";
 static const char *bad_index = "<BAD INDEX>";
 
-/*
- * The specification isn't very clear on endianness problems, so we better
- * have macros for these. It also helps us solve problems on systems that
- * don't support non-aligned memory access. This isn't a big issue IMHO,
- * since SMBIOS/DMI is intended mainly for Intel and compatible systems,
- * which are little-endian and support non-aligned memory access. Anyway,
- * you may use the following defines to control the way it works:
- * - Define BIGENDIAN on big-endian systems.
- * - Define ALIGNMENT_WORKAROUND if your system doesn't support
- *   non-aligned memory access. In this case, we use a slower, but safer,
- *   memory access method.
- * - If it happens that the table is supposed to be always little-endian
- *   ordered regardless of the architecture, define TABLE_LITTLEENDIAN.
- * You most probably will have to define none or the three of them.
- */
-
-#ifndef TABLE_LITTLEENDIAN
-#	ifdef BIGENDIAN
-	typedef struct {
-		u32 h;
-		u32 l;
-	} u64;
-#	else /* BIGENDIAN */
-	typedef struct {
-		u32 l;
-		u32 h;
-	} u64;
-#	endif /* BIGENDIAN */
-#	ifdef ALIGNMENT_WORKAROUND
-#		ifdef BIGENDIAN
-#		define WORD(x) (u16)((x)[1]+((x)[0]<<8))
-#		define DWORD(x) (u32)((x)[3]+((x)[2]<<8)+((x)[1]<<16)+((x)[0]<<24))
-#		define QWORD(x) (U64(DWORD(x+4), DWORD(x)))
-#		else /* BIGENDIAN */
-#		define WORD(x) (u16)((x)[0]+((x)[1]<<8))
-#		define DWORD(x) (u32)((x)[0]+((x)[1]<<8)+((x)[2]<<16)+((x)[3]<<24))
-#		define QWORD(x) (U64(DWORD(x), DWORD(x+4)))
-#		endif /* BIGENDIAN */
-#	else /* ALIGNMENT_WORKAROUND */
-#	define WORD(x) (u16)(*(u16 *)(x))
-#	define DWORD(x) (u32)(*(u32 *)(x))
-#	define QWORD(x) (*(u64 *)(x))
-#	endif /* ALIGNMENT_WORKAROUND */
-#else /* TABLE_LITTLEENDIAN */
-typedef struct {
-	u32 l;
-	u32 h;
-} u64;
-#define WORD(x) (u16)((x)[0]+((x)[1]<<8))
-#define DWORD(x) (u32)((x)[0]+((x)[1]<<8)+((x)[2]<<16)+((x)[3]<<24))
-#define QWORD(x) (U64(DWORD(x), DWORD(x+4)))
-#endif /* TABLE_LITTLEENDIAN */
-
-#if defined ALIGNMENT_WORKAROUND || defined TABLE_LITTLEENDIAN
-static u64 U64(u32 low, u32 high)
-{
-	u64 self;
-	
-	self.l=low;
-	self.h=high;
-	
-	return self;
-}
-#endif
-
 struct dmi_header
 {
 	u8 type;
@@ -141,7 +70,7 @@ struct dmi_header
 	u16 handle;
 };
 
-#if ((defined BIGENDIAN && defined TABLE_LITTLEENDIAN) || defined ALIGNMENT_WORKAROUND)
+#ifdef ALIGNMENT_WORKAROUND
 #define HANDLE(x) WORD((u8 *)&(x->handle))
 #else
 #define HANDLE(x) x->handle
@@ -2182,7 +2111,7 @@ static const char *dmi_memory_error_type(u8 code)
 	static const char *type[]={
 		"Other", /* 0x01 */
 		"Unknown",
-		"OK"
+		"OK",
 		"Bad Read",
 		"Parity Error",
 		"Single-bit Error",
@@ -2836,7 +2765,7 @@ static const char *dmi_power_supply_status(u8 code)
 		"Other", /* 0x01 */
 		"Unknown",
 		"OK",
-		"Non-critical"
+		"Non-critical",
 		"Critical" /* 0x05 */
 	};
 	
@@ -3790,57 +3719,24 @@ static void dmi_decode(u8 *data, u16 ver)
 	}
 }
 		
-static void dmi_table(int fd, u32 base, u16 len, u16 num, u16 ver, const char *pname, const char *devmem)
+static void dmi_table(u32 base, u16 len, u16 num, u16 ver, const char *devmem)
 {
 	u8 *buf;
 	u8 *data;
 	int i=0;
-#ifdef USE_MMAP
-	u32 mmoffset;
-	void *mmp;
-#endif /* USE_MMAP */
 	
 	printf("%u structures occupying %u bytes.\n",
 		num, len);
 	printf("Table at 0x%08X.\n",
 		base);
 	
-	if((buf=malloc(len))==NULL)
+	if((buf=mem_chunk(base, len, devmem))==NULL)
 	{
-		perror(pname);
-		return;
-	}
-#ifdef USE_MMAP
-	mmoffset=base%getpagesize();
-	/*
-	 * We were previously using PROT_WRITE and MAP_PRIVATE, but it caused
-	 * trouble once. So we are now mapping in read-only mode and copying
-	 * the interesting block into a regular memory buffer (similar to what
-	 * we do when not using mmap.)
-	 */
-	mmp=mmap(0, mmoffset+len, PROT_READ, MAP_SHARED, fd, base-mmoffset);
-	if(mmp==MAP_FAILED)
-	{
-		free(buf);
-    	perror(devmem);
-    	return;
-	}
-	memcpy(buf, (u8 *)mmp+mmoffset, len);
-	if(munmap(mmp, mmoffset+len)==-1)
-		perror(devmem);
-#else /* USE_MMAP */
-	if(lseek(fd, (off_t)base, SEEK_SET)==-1)
-	{
-		perror(devmem);
-		return;
-	}
-	if(myread(fd, buf, len, devmem)==-1)
-	{
-		free(buf);
+#ifndef USE_MMAP
 		printf("Table is unreachable, sorry. Try compiling dmidecode with -DUSE_MMAP.\n");
-		exit(1);
+#endif
+		return;
 	}
-#endif /* USE_MMAP */
 	
 	data=buf;
 	while(i<num && data+sizeof(struct dmi_header)<=buf+len)
@@ -3876,7 +3772,7 @@ static void dmi_table(int fd, u32 base, u16 len, u16 num, u16 ver, const char *p
 }
 
 
-static int smbios_decode(u8 *buf, int fd, const char *pname, const char *devmem)
+static int smbios_decode(u8 *buf, const char *devmem)
 {
 	if(checksum(buf, buf[0x05])
 	 && memcmp(buf+0x10, "_DMI_", 5)==0
@@ -3884,30 +3780,41 @@ static int smbios_decode(u8 *buf, int fd, const char *pname, const char *devmem)
 	{
 		printf("SMBIOS %u.%u present.\n",
 			buf[0x06], buf[0x07]);
-		dmi_table(fd, DWORD(buf+0x18), WORD(buf+0x16), WORD(buf+0x1C),
-			(buf[0x06]<<8)+buf[0x07], pname, devmem);
+		dmi_table(DWORD(buf+0x18), WORD(buf+0x16), WORD(buf+0x1C),
+			(buf[0x06]<<8)+buf[0x07], devmem);
 		return 1;
 	}
 	
 	return 0;
 }
 
+#ifndef USE_EFI
+static int legacy_decode(u8 *buf, const char *devmem)
+{
+	if(checksum(buf, 0x0F))
+	{
+		printf("Legacy DMI %u.%u present.\n",
+			buf[0x0E]>>4, buf[0x0E]&0x0F);
+		dmi_table(DWORD(buf+0x08), WORD(buf+0x06), WORD(buf+0x0C),
+			((buf[0x0E]&0xF0)<<4)+(buf[0x0E]&0x0F), devmem);
+		return 1;
+	}
+	
+	return 0;
+}
+#endif /* USE_EFI */
+
 int main(int argc, const char *argv[])
 {
-	int fd, found=0;
-	off_t fp=0xF0000;
+	int found=0;
+	off_t fp;
 	const char *devmem="/dev/mem";
 #ifdef USE_EFI
 	FILE *efi_systab;
+	const char *filename;
 	char linebuf[64];
-#ifdef USE_MMAP
-	u32 mmoffset;
-	void *mmp;
-#endif /* USE_MMAP */
 #endif /* USE_EFI */
-#if !(defined(USE_EFI) && defined(USE_MMAP))
-	u8 buf[0x20];
-#endif
+	u8 *buf;
 	
 	if(sizeof(u8)!=1 || sizeof(u16)!=2 || sizeof(u32)!=4 || '\0'!=0)
 	{
@@ -3917,20 +3824,21 @@ int main(int argc, const char *argv[])
 	
 	if(argc>=2)
 		devmem=argv[1];
-	if((fd=open(devmem, O_RDONLY))==-1)
-	{
-		perror(devmem);
-		exit(1);
-	}
 	
 	printf("# dmidecode %s\n", VERSION);
 	
 #ifdef USE_EFI
-	if((efi_systab=fopen("/proc/efi/systab", "r"))==NULL)
+	/*
+	 * Linux up to 2.6.6-rc2: /proc/efi/systab
+	 * Linux 2.6.6-rc3 and up: /sys/firmware/efi/systab
+	 */
+	if((efi_systab=fopen(filename="/proc/efi/systab", "r"))==NULL
+	&& (efi_systab=fopen(filename="/sys/firmware/efi/systab", "r"))==NULL)
 	{
-		perror("/proc/efi/systab");
+		perror(filename);
 		exit(1);
 	}
+	fp=0;
 	while((fgets(linebuf, sizeof(linebuf)-1, efi_systab))!=NULL)
 	{
 		char* addr=memchr(linebuf, '=', strlen(linebuf));
@@ -3942,83 +3850,42 @@ int main(int argc, const char *argv[])
 		}
 	}
 	if(fclose(efi_systab)!=0)
-		perror("/proc/efi/systab");
-
-#ifdef USE_MMAP
-	mmoffset=fp%getpagesize();
-	mmp=mmap(0, mmoffset+0x20, PROT_READ, MAP_PRIVATE, fd, fp-mmoffset);
-    if(mmp==MAP_FAILED)
-    {
-       perror(devmem);
-       exit(1);
-    }
-
-	smbios_decode(((u8 *)mmp)+mmoffset, fd, argv[0], devmem);
-
-	if(munmap(mmp, mmoffset+0x20)==-1)
-		perror(devmem);
-#else /* USE_MMAP */
-	if(lseek(fd, fp, SEEK_SET)==-1)
+		perror(filename);
+	if(fp==0)
 	{
-		perror(devmem);
+		fprintf(stderr, "%s: SMBIOS entry point missing\n", filename);
 		exit(1);
 	}
-	if(myread(fd, buf, 0x20, devmem)==-1)
-		exit(1);
 
-	smbios_decode(buf, fd, argv[0], devmem);
-#endif /* USE_MMAP */
-	found++;
-#else /* USE_EFI */
-	if(lseek(fd, fp, SEEK_SET)==-1)
-	{
-		perror(devmem);
+	if((buf=mem_chunk(fp, 0x20, devmem))==NULL)
 		exit(1);
-	}
-	while(fp<=0xFFFF0)
-	{
-		if(myread(fd, buf, 0x10, devmem)==-1)
-			exit(1);
-		fp+=16;
-		
-		if(memcmp(buf, "_SM_", 4)==0 && fp<=0xFFFF0)
-		{
-			if(myread(fd, buf+0x10, 0x10, devmem)==-1)
-				exit(1);
-			fp+=16;
-			
-			if(smbios_decode(buf, fd, argv[0], devmem))
-			{
-#ifndef USE_MMAP
-				/* dmi_table moved us far away */
-				lseek(fd, fp, SEEK_SET);
-#endif /* USE_MMAP */
-				found++;
-			}
-		}
-		else if(memcmp(buf, "_DMI_", 5)==0
-		 && checksum(buf, 0x0F))
-		{
-			printf("Legacy DMI %u.%u present.\n",
-				buf[0x0E]>>4, buf[0x0E]&0x0F);
-			dmi_table(fd, DWORD(buf+0x08), WORD(buf+0x06), WORD(buf+0x0C),
-				((buf[0x0E]&0xF0)<<4)+(buf[0x0E]&0x0F), argv[0], devmem);
-			
-#ifndef USE_MMAP
-			/* dmi_table moved us far away */
-			lseek(fd, fp, SEEK_SET);
-#endif /* USE_MMAP */
-			found++;
-		}
-	}
-#endif /* USE_EFI */
 	
-	if(close(fd)==-1)
-	{
-		perror(devmem);
+	if(smbios_decode(buf, devmem))
+		found++;
+	
+	free(buf);
+#else /* USE_EFI */
+	if((buf=mem_chunk(0xF0000, 0x10000, devmem))==NULL)
 		exit(1);
+	
+	for(fp=0; fp<=0xFFF0; fp+=16)
+	{
+		if(memcmp(buf+fp, "_SM_", 4)==0 && fp<=0xFFE0)
+		{
+			if(smbios_decode(buf+fp, devmem))
+				found++;
+			fp+=16;
+		}
+		else if(memcmp(buf+fp, "_DMI_", 5)==0)
+		{
+			if (legacy_decode(buf+fp, devmem))
+				found++;
+		}
 	}
 	
+	free(buf);
+#endif /* USE_EFI */
+	
 	if(!found)
 		printf("# No SMBIOS nor DMI entry point found, sorry.\n");
 
